@@ -158,8 +158,8 @@ class gui:
         while True:
             event, values = self.window.read(timeout=20)
 
-            values[
-                'Browse'] = '/home/chris/Dropbox (Partners HealthCare)/Gersten et al 2020 Fig 3 images/Fig7_max_projections adjusted/Carbo229_8kHz_63X_Maximum intensity projection.tif'
+            # values[
+            #     'Browse'] = '/home/chris/Dropbox (Partners HealthCare)/Gersten et al 2020 Fig 3 images/Fig7_max_projections adjusted/Carbo229_8kHz_63X_Maximum intensity projection.tif'
 
             self.rgb = [values['Red'], values['Green'], values['Blue']]
             self.contrast = [values['r_contrast'], values['g_contrast'], values['b_contrast']]
